@@ -4,7 +4,7 @@
  * Defines a very simple command line argument parser.
  *
  * @copyright
- * Copyright (c) 2017 Viet The Nguyen
+ * Copyright (c) 2018 Viet The Nguyen
  *
  * @copyright
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -31,16 +31,11 @@
 #ifndef ARGAGG_ARGAGG_ARGAGG_HPP
 #define ARGAGG_ARGAGG_ARGAGG_HPP
 
-#ifdef __unix__
-#include <stdio.h>
-#include <unistd.h>
-#endif // #ifdef __unix__
-
 #include <algorithm>
 #include <array>
+#include <cctype>
 #include <cstdlib>
 #include <cstring>
-#include <cctype>
 #include <iterator>
 #include <ostream>
 #include <sstream>
@@ -56,8 +51,8 @@
  * There are only two hard things in Computer Science: cache invalidation and
  * naming things (Phil Karlton).
  *
- * The names of types have to be succint and clear. This has turned out to be a
- * more difficult thing than I expected. Here you'll find a quick overview of
+ * The names of types have to be succinct and clear. This has turned out to be
+ * a more difficult thing than I expected. Here you'll find a quick overview of
  * the type names you'll find in this namespace (and thus "library").
  *
  * When a program is invoked it is passed a number of "command line arguments".
@@ -140,7 +135,7 @@ struct option_lacks_argument_error
  * @brief
  * This exception is thrown when an option's flag is invalid. This can be the
  * case if the flag is not prefixed by one or two hyphens or contains non
- * alpha-numeric characters after the hypens. See is_valid_flag_definition()
+ * alpha-numeric characters after the hyphens. See is_valid_flag_definition()
  * for more details.
  */
 struct invalid_flag
@@ -149,6 +144,17 @@ struct invalid_flag
 };
 
 
+/**
+ * @brief
+ * This exception is thrown when an unknown option is requested by name from an
+ * argagg::parser_results through the indexing operator ([]).
+ */
+struct unknown_option
+: public std::runtime_error {
+  using std::runtime_error::runtime_error;
+};
+
+
 /**
  * @brief
  * The set of template instantiations that convert C-strings to other types for
@@ -165,6 +171,86 @@ namespace convert {
   template <typename T>
   T arg(const char* arg);
 
+  /**
+   * @brief
+   * For simple types the main extension point for adding argument conversions
+   * is argagg::convert::arg<T>(). However, for complex types such as templated
+   * types partial specialization of a helper struct is required. This struct
+   * provides that extension point. The default, generic implementation of
+   * argagg::convert::arg<T>() calls converter<T>::convert().
+   *
+   * @see
+   * @ref argagg::csv
+   */
+  template <typename T>
+  struct converter {
+    static T convert(const char* arg);
+  };
+
+  /**
+   * @brief
+   * A utility function for parsing an argument as a delimited list. To use,
+   * initialize a const char* pointer to the start of argument string. Then
+   * call parse_next_component(), providing that pointer, a mutable reference
+   * to where the parsed argument will go, and optionally the delimiting
+   * character. The argument string will be read up to the next delimiting
+   * character and then converted using
+   * <tt>argagg::convert::arg<decltype(out_arg)>()</tt>. The pointer is then
+   * incremented accordingly. If the delimiting character is no longer found
+   * then false is returned meaning that parsing the list can be considered
+   * finished.
+   *
+   * @code
+     #include <argagg/argagg.hpp>
+
+     struct position3 {
+       double x;
+       double y;
+       double z;
+     };
+
+     namespace argagg {
+     namespace convert {
+       template <>
+       position3 arg(const char* s)
+       {
+         position3 result {0.0, 0.0, 0.0};
+         if (!parse_next_component(s, result.x)) {
+           // could potentially throw an error if you require that at least two
+           // components exist in the list
+           return result;
+         }
+         if (!parse_next_component(s, result.y)) {
+           return result;
+         }
+         if (!parse_next_component(s, result.z)) {
+           return result;
+         }
+         return result;
+       }
+     } // namespace convert
+     } // namespace argagg
+
+     int main(int argc, char** argv)
+     {
+       argagg::parser argparser {{
+          { "origin", {"-o", "--origin"},
+            "origin as position3 specified as a comma separated list of "
+            "components (e.g. '1,2,3')", 1},
+        }};
+       argagg::parser_results args = argparser.parse(argc, argv);
+       auto my_position = args["origin"].as<position3>();
+       // ...
+       return 0;
+     }
+     @endcode
+   */
+  template <typename T>
+  bool parse_next_component(
+    const char*& s,
+    T& out_arg,
+    const char delim = ',');
+
 }
 
 
@@ -223,6 +309,13 @@ struct option_result {
   template <typename T>
   operator T () const;
 
+  /**
+   * @brief
+   * Explicitly define a unary not operator that wraps the implicit boolean
+   * conversion specialization in case the compiler can't do it automatically.
+   */
+  bool operator ! () const;
+
 };
 
 
@@ -300,6 +393,13 @@ struct option_results {
   template <typename T>
   operator T () const;
 
+  /**
+   * @brief
+   * Explicitly define a unary not operator that wraps the implicit boolean
+   * conversion specialization in case the compiler can't do it automatically.
+   */
+  bool operator ! () const;
+
 };
 
 
@@ -568,25 +668,17 @@ struct parser {
 /**
  * @brief
  * A convenience output stream that will accumulate what is streamed to it and
- * then, on destruction, format the accumulated string using the fmt program
- * (via the argagg::fmt_string() function) to the provided std::ostream.
+ * then, on destruction, format the accumulated string (via the
+ * argagg::fmt_string() function) to the provided std::ostream.
  *
  * Example use:
  *
  * @code
  * {
  *   argagg::fmt_ostream f(std::cerr);
- *   f << "Usage: " << really_long_string << std::endl;
+ *   f << "Usage: " << really_long_string << '\n';
  * } // on destruction here the formatted string will be streamed to std::cerr
  * @endcode
- *
- * @note
- * This only has formatting behavior if the <tt>__unix__</tt> preprocessor
- * definition is defined since formatting relies on the POSIX API for forking,
- * executing a process, and reading/writing to/from file descriptors. If that
- * preprocessor definition is not defined then this class has the same overall
- * behavior except the output string is not formatted (basically streams
- * whatever the accumulated string is). See arggg::fmt_string().
  */
 struct fmt_ostream : public std::ostringstream {
 
@@ -617,18 +709,9 @@ struct fmt_ostream : public std::ostringstream {
 
 /**
  * @brief
- * Processes the provided string using the fmt util and returns the resulting
- * output as a string. Not the most efficient (in time or space) but gets the
- * job done.
- *
- * This function is cowardly so if there are any errors encountered such as a
- * syscall returning -1 then the input string is returned.
- *
- * @note
- * This only has formatting behavior if the <tt>__unix__</tt> preprocessor
- * definition is defined since it relies on the POSIX API for forking,
- * executing a process, reading/writing to/from file descriptors, and the
- * existence of the fmt util.
+ * Processes the provided string using the fmt utility and returns the
+ * resulting output as a string. Not the most efficient (in time or space) but
+ * gets the job done.
  */
 std::string fmt_string(const std::string& s);
 
@@ -694,6 +777,13 @@ option_result::operator bool () const
 }
 
 
+inline
+bool option_result::operator ! () const
+{
+  return !static_cast<bool>(*this);
+}
+
+
 inline
 std::size_t option_results::count() const
 {
@@ -749,6 +839,13 @@ option_results::operator bool () const
 }
 
 
+inline
+bool option_results::operator ! () const
+{
+  return !static_cast<bool>(*this);
+}
+
+
 inline
 bool parser_results::has_option(const std::string& name) const
 {
@@ -759,16 +856,24 @@ bool parser_results::has_option(const std::string& name) const
 
 inline
 option_results& parser_results::operator [] (const std::string& name)
-{
+try {
   return this->options.at(name);
+} catch (const std::out_of_range& e) {
+  std::ostringstream msg;
+  msg << "no option named \"" << name << "\" in parser_results";
+  throw unknown_option(msg.str());
 }
 
 
 inline
 const option_results&
 parser_results::operator [] (const std::string& name) const
-{
+try {
   return this->options.at(name);
+} catch (const std::out_of_range& e) {
+  std::ostringstream msg;
+  msg << "no option named \"" << name << "\" in parser_results";
+  throw unknown_option(msg.str());
 }
 
 
@@ -968,7 +1073,7 @@ inline
 bool parser_map::known_short_flag(
   const char flag) const
 {
-  return this->short_map[flag] != nullptr;
+  return this->short_map[static_cast<std::size_t>(flag)] != nullptr;
 }
 
 
@@ -976,7 +1081,7 @@ inline
 const definition* parser_map::get_definition_for_short_flag(
   const char flag) const
 {
-  return this->short_map[flag];
+  return this->short_map[static_cast<std::size_t>(flag)];
 }
 
 
@@ -1026,8 +1131,9 @@ parser_map validate_definitions(
       }
 
       if (flag_is_short(flag.data())) {
-        const int short_flag_letter = flag[1];
-        const auto existing_short_flag = map.short_map[short_flag_letter];
+        const std::size_t short_flag_letter = static_cast<std::size_t>(flag[1]);
+        const auto existing_short_flag =
+          map.short_map[short_flag_letter];
         bool short_flag_already_exists = (existing_short_flag != nullptr);
         if (short_flag_already_exists) {
           std::ostringstream msg;
@@ -1036,7 +1142,7 @@ parser_map validate_definitions(
               << "\" and option \"" << existing_short_flag->name;
           throw invalid_flag(msg.str());
         }
-        map.short_map[short_flag_letter] = &defn;
+        map.short_map[static_cast<std::size_t>(short_flag_letter)] = &defn;
         continue;
       }
 
@@ -1070,7 +1176,7 @@ parser_results parser::parse(int argc, const char** argv) const
   // turning the parser into a thin interface (by re-exposing setters and
   // getters) to the vector methods just so that I can catch when the
   // definition has been modified. It seems much simpler to just enforce the
-  // validity when you actually want to parser because it's at the moment of
+  // validity when you actually want to parse because it's at the moment of
   // parsing that you know the definitions are complete.
   parser_map map = validate_definitions(this->definitions);
 
@@ -1168,7 +1274,7 @@ parser_results parser::parse(int argc, const char** argv) const
       auto long_flag_arg = std::strchr(arg_i_cstr, '=');
       std::size_t flag_len = arg_i_len;
       if (long_flag_arg != nullptr) {
-        flag_len = long_flag_arg - arg_i_cstr;
+        flag_len = static_cast<std::size_t>(long_flag_arg - arg_i_cstr);
       }
       std::string long_flag_str(arg_i_cstr, flag_len);
 
@@ -1378,6 +1484,13 @@ namespace convert {
 #undef DEFINE_CONVERSION_FROM_LONG_LONG_
 
 
+  template <typename T>
+  T arg(const char* arg)
+  {
+    return converter<T>::convert(arg);
+  }
+
+
   template <> inline
   bool arg(const char* arg)
   {
@@ -1434,7 +1547,29 @@ namespace convert {
     return std::string(arg);
   }
 
-}
+
+  template <typename T>
+  bool parse_next_component(
+    const char*& s,
+    T& out_arg,
+    const char delim)
+  {
+    const char* begin = s;
+    s = std::strchr(s, delim);
+    if (s == nullptr) {
+      std::string arg_str(begin);
+      out_arg = argagg::convert::arg<T>(arg_str.c_str());
+      return false;
+    } else {
+      std::string arg_str(begin, static_cast<std::size_t>(s - begin));
+      out_arg = argagg::convert::arg<T>(arg_str.c_str());
+      s += 1;
+      return true;
+    }
+  }
+
+
+} // namespace convert
 
 
 inline
@@ -1451,79 +1586,105 @@ fmt_ostream::~fmt_ostream()
 }
 
 
-#ifdef __unix__
+inline
+std::string lstrip(const std::string& text)
+{
+  auto result = text;
+
+  result.erase(
+    result.begin(),
+    std::find_if(
+      result.begin(),
+      result.end(),
+      [](int ch) { return !std::isspace(ch); }));
+
+  return result;
+}
 
 
 inline
-std::string fmt_string(const std::string& s)
+std::string rstrip(const std::string& text)
 {
-  constexpr int read_end = 0;
-  constexpr int write_end = 1;
+  auto result = text;
 
-  // TODO (vnguyen): This function overall needs to handle possible error
-  // returns from the various syscalls.
+  result.erase(
+    std::find_if(
+      result.rbegin(),
+      result.rend(),
+      [](int ch) { return !std::isspace(ch); }).base(),
+    result.end());
 
-  int read_pipe[2];
-  int write_pipe[2];
-  if (pipe(read_pipe) == -1) {
-    return s;
-  }
-  if (pipe(write_pipe) == -1) {
-    return s;
+  return result;
+}
+
+
+inline
+std::string construct_line(const std::string& indent,
+                           const std::string& contents)
+{
+  return indent + rstrip(contents) + "\n";
+}
+
+
+/**
+ * @brief
+ * Return a wrapped version of a single line of text.
+ */
+inline
+std::string wrap_line(const std::string& single_line,
+                      const std::size_t wrap_width)
+{
+  auto indentation_spaces = single_line.find_first_not_of(" ");
+  if (indentation_spaces == std::string::npos) {
+    indentation_spaces = 0;
   }
 
-  auto parent_pid = fork();
-  bool is_fmt_proc = (parent_pid == 0);
-  if (is_fmt_proc) {
-    dup2(write_pipe[read_end], STDIN_FILENO);
-    dup2(read_pipe[write_end], STDOUT_FILENO);
-    close(write_pipe[read_end]);
-    close(write_pipe[write_end]);
-    close(read_pipe[read_end]);
-    close(read_pipe[write_end]);
-    const char* argv[] = {"fmt", NULL};
-    execvp(const_cast<char*>(argv[0]), const_cast<char**>(argv));
-  }
+  const auto line = lstrip(single_line);
+  const auto indent = std::string(indentation_spaces, ' ');
 
-  close(write_pipe[read_end]);
-  close(read_pipe[write_end]);
-  auto fmt_write_fd = write_pipe[write_end];
-  auto write_result = write(fmt_write_fd, s.c_str(), s.length());
-  if (write_result != static_cast<ssize_t>(s.length())) {
-    return s;
-  }
-  close(fmt_write_fd);
+  std::string result;
 
-  auto fmt_read_fd = read_pipe[read_end];
-  std::ostringstream os;
-  char buf[64];
+  std::size_t position = 0;
+  std::size_t line_start = 0;
   while (true) {
-    auto read_count = read(
-      fmt_read_fd, reinterpret_cast<void*>(buf), sizeof(buf));
-    if (read_count <= 0) {
+    const auto new_position = line.find_first_of(" ", position);
+    if (new_position == std::string::npos) {
       break;
     }
-    os.write(buf, static_cast<std::streamsize>(read_count));
+
+    if (new_position + indentation_spaces > line_start + wrap_width) {
+      result += construct_line(
+        indent, line.substr(line_start, position - line_start - 1));
+
+      line_start = position;
+    }
+
+    position = new_position + 1;
   }
-  close(fmt_read_fd);
 
-  return os.str();
+  return result + construct_line(indent, line.substr(line_start));
 }
 
 
-#else // #ifdef __unix__
-
-
 inline
 std::string fmt_string(const std::string& s)
 {
-  return s;
+  std::stringstream ss(s);
+  std::string line;
+
+  std::string result;
+
+  // Use default width of `fmt`.
+  const auto column_width = 75;
+
+  while (std::getline(ss, line, '\n')) {
+    result += wrap_line(line, column_width);
+  }
+
+  return result;
 }
 
 
-#endif // #ifdef __unix__
-
-
 } // namespace argagg
 
 
@@ -1538,8 +1699,7 @@ std::ostream& operator << (std::ostream& os, const argagg::parser& x)
         os << ", ";
       }
     }
-    os << std::endl;
-    os << "        " << definition.help << std::endl;
+    os << "\n        " << definition.help << '\n';
   }
   return os;
 }
